﻿using System.Collections.Generic;
using System.Text;
using Sdl.FileTypeSupport.Framework.BilingualApi;
using Sdl.FileTypeSupport.Framework.NativeApi;

namespace CustomWordCounter.Content
{
    public class ContentReader : IMarkupDataVisitor
    {

        public ContentReader()
        {
            Initialize();
        }

        private void Initialize()
        {
            Segment = new Sdl.LanguagePlatform.Core.Segment();
            PlainText = new StringBuilder("");
            Comments  = new List<IComment>();
            TagCounter = 0;
        }
        private int TagCounter { get; set; }
        public List<IComment> Comments { get; set; }
        public StringBuilder PlainText { get; set; }
       
        public Sdl.LanguagePlatform.Core.Segment Segment { get; set; }

        public bool IncludeTagContent { get; set; }

        public bool IgnoreComments { get; set; }

        public string GetPlainText(ISegment segment, bool includeTagContent)
        {
         
            IgnoreComments = true;
            IncludeTagContent = includeTagContent;
            Initialize(); 
            VisitChildren(segment);
            return PlainText.ToString();
        }

        public List<IComment> GetSegmentComments(ISegment segment)
        {
            IgnoreComments = false;
            Initialize();
            VisitChildren(segment);
            return Comments;
        }

        public void ProcessSegment(ISegment segment)
        {
            Initialize();
            VisitChildren(segment);
        }

        private void VisitChildren(IAbstractMarkupDataContainer container)
        {
            if (container == null)
                return;
            foreach (var item in container)
            {
                item.AcceptVisitor(this);                
            }
        }

        public void VisitCommentMarker(ICommentMarker commentMarker)
        {
            if (!IgnoreComments)
            {
                foreach (var commentProperty in commentMarker.Comments.Comments)
                {
                    Comments.Add(commentProperty);
                }
            }
            VisitChildren(commentMarker);
        }

        public void VisitLocationMarker(ILocationMarker location)
        {            
            // Not required for this implementation
        }

        public void VisitLockedContent(ILockedContent lockedContent)
        {            
            PlainText.Append(lockedContent.Content);

            var lockedTag = new Sdl.LanguagePlatform.Core.Tag
            {
                Type = Sdl.LanguagePlatform.Core.TagType.LockedContent,
                TextEquivalent = lockedContent.Content.ToString(),
                Anchor = TagCounter++
            };
            Segment.Add(lockedTag);
        }

        public void VisitOtherMarker(IOtherMarker marker)
        {
            VisitChildren(marker);
        }

        public void VisitPlaceholderTag(IPlaceholderTag tag)
        {
            if (IncludeTagContent)
                PlainText.Append("<" + tag.Properties.TagContent + "/>");

            var placeHolderTag = new Sdl.LanguagePlatform.Core.Tag
            {
                Type = Sdl.LanguagePlatform.Core.TagType.TextPlaceholder,
                TagID = tag.TagProperties.TagId.Id,
                TextEquivalent = tag.Properties.TextEquivalent,
                Anchor = TagCounter++
            };
            Segment.Add(placeHolderTag);
        }

        public void VisitRevisionMarker(IRevisionMarker revisionMarker)
        {
            VisitChildren(revisionMarker);
        }

        public void VisitSegment(ISegment segment)
        {
            VisitChildren(segment);
        }

        public void VisitTagPair(ITagPair tagPair)
        {
            if (IncludeTagContent)
                PlainText.Append("<" + tagPair.StartTagProperties.TagContent + ">");

            var tgStart = new Sdl.LanguagePlatform.Core.Tag
            {
                Type = Sdl.LanguagePlatform.Core.TagType.Start,
                Anchor = TagCounter,
                TagID = tagPair.StartTagProperties.TagId.Id,
                TextEquivalent = tagPair.StartTagProperties.DisplayText
            };
            Segment.Add(tgStart);

            VisitChildren(tagPair);

            if (IncludeTagContent)
                PlainText.Append("</" + tagPair.EndTagProperties.TagContent + ">");

            var tgEnd = new Sdl.LanguagePlatform.Core.Tag
            {
                Type = Sdl.LanguagePlatform.Core.TagType.End,
                TagID = tagPair.StartTagProperties.TagId.Id,
                Anchor = TagCounter++,
                TextEquivalent = tagPair.EndTagProperties.DisplayText
            };
            Segment.Add(tgEnd);
        }

        public void VisitText(IText text)
        {
            PlainText.Append(text.Properties.Text);

            Segment.Add(text.Properties.Text);
        }
    }
}
