﻿using System.IO;
using Sdl.FileTypeSupport.Framework.BilingualApi;
using Sdl.LanguagePlatform.Core;
using Sdl.LanguagePlatform.Core.Tokenization;
using Sdl.LanguagePlatform.TranslationMemory;
using Sdl.LanguagePlatform.TranslationMemoryApi;

namespace CustomWordCounter.Content
{
    public class TemporaryAnalysisTm
    {
        private FileBasedTranslationMemory MyAnalysisTM { get; set; }

        private CustomPaths _customPathInfo;
        public CustomPaths CustomPathInfo => _customPathInfo ?? (_customPathInfo = new CustomPaths());

        public SearchResults SearchTranslationMemory(Segment segment)
        {
            return MyAnalysisTM == null 
                ? null 
                : MyAnalysisTM.LanguageDirection.SearchSegment(GetSearchSettings(), segment);
        }

        public void CreateTranslationMemory(IDocumentProperties documentInfo)
        {
            var tmName = "TM."
                         + documentInfo.SourceLanguage.CultureInfo.Name
                         + "-"
                         + documentInfo.TargetLanguage.CultureInfo.Name
                         + ".sdltm";

            MyAnalysisTM = new FileBasedTranslationMemory(Path.Combine(CustomPathInfo.MyCustomTmPath, tmName)
                , "Temporary analysis TM"
                , documentInfo.SourceLanguage.CultureInfo
                , documentInfo.TargetLanguage.CultureInfo
                , FuzzyIndexes.SourceWordBased
                , new BuiltinRecognizers()
                , TokenizerFlags.DefaultFlags
                , WordCountFlags.DefaultFlags
                , false);
            MyAnalysisTM.FieldDefinitions.Add(new FieldDefinition("FileIndex", FieldValueType.Integer));
            MyAnalysisTM.Save();

            
        }

        private static SearchSettings GetSearchSettings()
        {
            var settings = new SearchSettings
            {
                MaxResults = 1,
                MinScore = 100,
                Mode = SearchMode.ExactSearch
            };
            return settings;
        }

    }
}
